# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl OOPerlTest-Personal.t'

#########################

use Test::More tests => 9;


my $class_workUnit = 'OOPerlTest::Personal::WorkUnit';

# 1: can we load the class?
use_ok($class_workUnit);


{
    # create some work
    my @workUnits =
        map
        {
            $class_workUnit->new
            (
                project => 'Making doughnuts',
                task => 'cook batch ' . $_,
                status => 'NotStarted',
                payload => 'test',
            )
        } (1 .. 50);

    # check types
    # 2: are all our work units of the right type?
    is
    (
        scalar(grep { $_->isa($class_workUnit) } @workUnits), scalar(@workUnits),
        ,'WorkUnits are of correct type.'
    );

    # check statuses
    eval
    {
        map { $workUnits[0]->status($_) } qw(Completed Started NotStarted Rejected)
    };

    # 3: are all the allowed statuses allowed?
    ok
    (
        !$@,
        'Allowed statuses worked.'
    );

    eval
    {
        $workUnits[0]->status('InvalidStatus');
    };

    # 4: are disallowed statuses not allowed?
    like
    (
        $@, qr/InvalidStatus/,
        'Invalid status generates exception.'
    );

    eval
    {
        $class_workUnit->new
        (
            task => 'cook batch',
            status => 'NotStarted',
            payload => 'test',
        )
    };

    # 5: is the project attribute required?
    like
    (
        $@, qr/project/,
        'WorkUnit enforces required project attribute.',
    );

    eval
    {
        $class_workUnit->new
        (
            project => 'Making doughnuts',
            status => 'NotStarted',
            payload => 'test',
        )
    };

    # 6: is the task attribute required?
    like
    (
        $@, qr/task/,
        'WorkUnit enforces required task attribute.',
    );


    eval
    {
        $class_workUnit->new
        (
            project => 'Making doughnuts',
            task => 'cook batch',
            payload => 'test',
        )
    };

    # 7: is the status attribute required?
    like
    (
        $@, qr/status/,
        'WorkUnit enforces required status attribute.',
    );


    eval
    {
        $class_workUnit->new
        (
            project => 'Making doughnuts',
            task => 'cook batch',
            status => 'NotStarted',
        )
    };

    # 8: is the payload attribute optional?
    ok
    (
        !$@,
        'WorkUnit->payload is optional attribute.'
    );


    eval
    {
        $class_workUnit->new
        (
            project => 'Making doughnuts',
            task => 'cook batch',
            status => 'NotStarted',
            invalidAttribute => 'hehe',
        )
    };

    # 9: are extra (unknown) attributes caught as an error?
    like
    (
        $@, qr/invalidAttribute/,
        'Bonus: WorkUnit generates exception with invalid attribute.',
    );

}


