# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl OOPerlTest-Personal.t'

#########################

use Test::More tests => 22;


my $class_person = 'OOPerlTest::Personal::Person';
my $class_workUnit = 'OOPerlTest::Personal::WorkUnit';
my $class_employee = 'OOPerlTest::Personal::Person::Employee';
eval qq{ use $class_workUnit };

# 1: can we load our class?
use_ok($class_employee);


# testing positive
{

    # 2: are all attributes (including inherited ones) available as methods?
    can_ok
    (
        $class_employee,
        (
            qw(
                name
                alias
                dateOfBirth
                hireDate
            )
        )
    );

    my $employee = $class_employee->new
    (
        name => 'Benigno Millares',
        alias => 'benny',
        dateOfBirth => '10/24/1970',
        hireDate => '01/01/2010',
    );

    # 3,4: does our object have the right type (including inheritance)?
    isa_ok $employee, $class_employee, 'new Employee';
    isa_ok $employee, $class_person, 'new Employee';

    # 5: is the name attribute set on creation? (also tests accessor)
    is
    (
        $employee->name, 'Benigno Millares',
        'Employee->name holds its value'
    );


    # 6: is the alias attribute set on creation? (also tests accessor)
    is
    (
        $employee->alias, 'benny',
        'Employee->alias holds its value'
    );

    # 7: is the dateOfBirth attribute set on creation? (also tests accessor)
    is
    (
        $employee->dateOfBirth, '10/24/1970',
        'Employee->dateOfBirth holds its value'
    );

    # 8: is the hireDate attribute set on creation? (also tests accessor)
    is
    (
        $employee->hireDate, '01/01/2010',
        'Employee->hireDate holds its value'
    );

}


{
    my $employee = $class_employee->new
    (
        name => 'Benigno Millares',
        alias => 'benny',
        dateOfBirth => '10/24/1970',
        hireDate => '01/01/2010',
    );

    # changing it to something else after instantiating object
    $employee->hireDate('07/01/2010');

    # 9: does the mutator for the hireDate attribute work?
    is
    (
        $employee->hireDate, '07/01/2010',
        'Changing Employee->name'
    );

}


# testing negatives
{
    my $employee = eval
    {
        $class_employee->new
        (
            name => 'Benigno Millares',
            alias => 'benny',
            dateOfBirth => '10/24/1970',
        );
    };

    # 10: is the hireDate attribute required?
    ok
    (
        $@,
        'Employee->hireDate required attribute exception.'
    );
}


{
    my $employee = $class_employee->new
    (
        name => 'Benigno Millares',
        alias => 'benny',
        dateOfBirth => '10/24/1970',
        hireDate => '01/01/2010',
    );

    my $person = $employee->Quit;


    # 11: after quitting, our Employee should be a Person
    isa_ok
    (
        $person, $class_person,
        'Employee->Quit return value'
    );

    # 12: of course, they were a Person before too ... are they no longer an Employee?
    ok
    (
        !$person->isa($class_employee),
        'Employee after quitting is no longer an Employee'
    );

    # 13,14,15: quitting should not change the basic attributes of the person
    is $person->name, 'Benigno Millares', 'Employee->Quit return value attributes (name)';
    is $person->alias, 'benny', 'Employee->Quit return value attributes (alias)';
    is $person->dateOfBirth, '10/24/1970', 'Employee->Quit return value attributes (dateOfBirth)';

    eval { $person->Work };

    # 16: a Person who isn't an Employee shouldn't be able to Work
    ok
    (
        $@,
        'Bonus: Employee should generate exception if Work is called after Quit.'
    );
}


{
    my $employee = $class_employee->new
    (
        name => 'Benigno Millares',
        alias => 'benny',
        dateOfBirth => '10/24/1970',
        hireDate => '01/01/2010'
    );


    my $workUnit = $class_workUnit->new
    (
        project => 'New website',
        task => 'Create session class',
        status => 'NotStarted',
        payload => <<END,
01/13/2009 some event occurred
04/29/2009 some other event occurred
Kilroy was here!
an unusual event occurred on 06/09/2009
10/31/2009 was Halloween
END
    );

    # 17: basic sanity check (work unit status)
    is
    (
        $workUnit->status, 'NotStarted',
        'WorkUnit inital status'
    );

    $workUnit = $employee->Work($workUnit);

    # 18: does performing Work cause a work unit to be Completed?
    is
    (
        $workUnit->status, 'Completed',
        'WorkUnit status after returned by method Work'
    );

    # 19: is our payload changed in the way we expect?
    is
    (
        $workUnit->payload, <<END,
20090113 some event occurred
20090429 some other event occurred
Kilroy was here!
an unusual event occurred on 06/09/2009
20091031 was Halloween
END
        'Payload processed according to specs.'
    );

    $workUnit->status('Rejected');

    $workUnit = $employee->Work($workUnit);

    # 20: performing Work on a Rejected work unit should not change its status
    is
    (
        $workUnit->status, 'Rejected',
        'WorkUnit status after Rejected WorkUnit returned by method Work'
    );

    $workUnit = $class_workUnit->new
    (
        project => 'New website',
        task => 'Create session class',
        status => 'NotStarted',
        payload => <<END,
20090113 some event occurred
20090429 some other event occurred
Kilroy was here!
an unusual event occurred on 06/09/2009
20091031 was Halloween
END
    );

    $workUnit = $employee->Work($workUnit);

    # 21: if the payload requires no modification, is the work unit Rejected?
    is
    (
        $workUnit->status, 'Rejected',
        'WorkUnit status after a WorkUnit with a payload that should not be processed.'
    );
}


{
    my $employee = eval
    {
        $class_employee->new
        (
            name => 'Benigno Millares',
            alias => 'benny',
            dateOfBirth => '10/24/1970',
            hireDate => '10/23/1969',
        );
    };

    # 22: probably shouldn't hire people who haven't been born yet :-)
    ok
    (
        $@,
        'Bonus: Employee->hireDate should be greater than dateOfBirth.'
    );
}


