package OOPerlTest::Personal::Person::Employee;

use Moose;
use Moose::Util::TypeConstraints;

extends 'OOPerlTest::Personal::Person';

subtype 'HDate',
   as 'Str',
   where {  my ($month, $day, $year) = m|(\d\d)/(\d\d)/(\d{4})|;
             $month ||= 0;
             $day ||= 0;
             !($month < 1 || $month > 12 || $day < 1 || $day > 32); },
   message { "Invalid hireDate" };

has 'hireDate' => (isa => 'HDate', is => 'rw', required => 1);

sub Work {
  my ($self, $workUnit) = @_;
  
  my $payload = $workUnit->payload();
  
  my $changes;
  
  if (defined $payload) {
    $changes = ($payload =~ s|^(\d\d)/(\d\d)/(\d{4})\b|$3$1$2|mg);
  }

  # if we had changes, then it's Completed, otherwise it's Rejected
  my $status = $changes ? "Completed" : "Rejected";
  
  # update the workUnit with the new status and payload
  $workUnit->payload($payload);
  $workUnit->status($status);
  
  return $workUnit;
}
sub Quit { 
  my ($self) = @_;
  
  my $person = OOPerlTest::Personal::Person->new({
     name => $self->name(),
     ($self->alias() ? (alias => $self->alias()) : ()),
     dateOfBirth => $self->dateOfBirth(),
  });
  return $person;
}

sub checkattr {
  my $self = shift;
  my $args = shift;

  for my $attr (keys %$args) {
    if ($attr ne "dateOfBirth" && $attr ne "alias" && $attr ne "name" && $attr ne "hireDate") {
      die "invalidAttribute";
    }
  }
}

  sub BUILD {
      my $self = shift;
      my $args = shift;
      
      $self->checkattr($args);
            
      # check if people are hired before they were born.
      my ($hmonth, $hday, $hyear) = ($self->hireDate()    =~ m|(\d\d)/(\d\d)/(\d{4})|);
      my ($bmonth, $bday, $byear) = ($self->dateOfBirth() =~ m|(\d\d)/(\d\d)/(\d{4})|);
      
      if ($byear > $hyear) {
        die "dateOfBirth occurs after hireDate";
      } elsif ($byear == $hyear) { # hiring babies, someone repealed the child labor laws.
        if ($bmonth > $hmonth) {
          die "dateOfBirth occurs after hireDate";
        } elsif ($bmonth == $hmonth) {  # must be some wild times for this to happen
          if ($bday > $hday) {
            die "dateOfBirth occurs after hireDate";
          }
        } 
      }
  }

1;
