package OOPerlTest::Personal::Person::Employee::Manager;

use Moose;
use POSIX qw(strftime);
use Data::Dumper;

extends 'OOPerlTest::Personal::Person::Employee';

has 'employees' => (is => 'rw', isa => 'ArrayRef[OOPerlTest::Personal::Person::Employee]', 'default' => sub{[]});
has 'workUnitsCompleted' => (is => 'rw', isa => 'Int', default => sub {0});
has 'workUnitsRejected' => (is => 'rw', isa => 'Int', default => sub {0});

sub Hire {
  my ($self, $person) = @_;
  
  # get today's date
  my $hiredate = strftime('%M/%D/%Y', localtime());
  
  my $empl = OOPerlTest::Personal::Person::Employee->new({
    name => $person->name(),
    dateOfBirth => $person->dateOfBirth(),
    hireDate => $hiredate,
    ($person->alias() ? (alias => $person->alias()) : ())
  });
  
  push @{$self->employees()}, $empl; 
  
  return $empl;
}

sub Fire {
  my ($self, $empl) = @_;
  
  # get rid of the employee from the list
  my @empls = grep {$_ ne $empl} @{$self->employees};
  $self->employees(\@empls);
  
  my $person = OOPerlTest::Personal::Person->new({
     name => $empl->name(),
     ($empl->alias() ? (alias => $empl->alias()) : ()),
     dateOfBirth => $empl->dateOfBirth(),
  });
  return $person;
}

sub Work {
  my ($self, $work) = @_;
  
  # don't work if it's already done
  return $work if ($work->status() eq "Completed" || $work->status() eq "Rejected");
  
  # this will naively pick the first person always to work.  any real system should distrubute the work
  if (@{$self->employees()} >= 1) {
    my $newwork = ${$self->employees()}[0]->Work($work);
    
    if ($newwork->status() eq "Completed") {
      $self->workUnitsCompleted(1+$self->workUnitsCompleted());
    } elsif ($newwork->status() eq "Rejected") {
      $self->workUnitsRejected(1+$self->workUnitsRejected());
    } else {
      die "Failed to finish the work";
    }
    
    return $newwork;
  } else {
    die "Can't work without any employees"
  } 

}

sub Quit {
  my ($self) = @_;
  
  my $empl = OOPerlTest::Personal::Person::Employee->new({
     name => $self->name(),
     ($self->alias() ? (alias => $self->alias()) : ()),
     dateOfBirth => $self->dateOfBirth(),
     hireDate => $self->hireDate(),
  });
  return $empl;
}

sub WorkUnitsProcessed {
  my ($self) = @_;
  return "Completed: ".$self->workUnitsCompleted().", Rejected: ".$self->workUnitsRejected;
}

1;
