# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl OOPerlTest-Personal.t'

#########################

use Test::More tests => 15;


my $class_person = 'OOPerlTest::Personal::Person';

# 1: can we load the class?
use_ok($class_person);


# testing positive
{

    # 2: are all attributes available as methods?
    can_ok
    (
        $class_person,
        (
            qw(
                name
                alias
                dateOfBirth
            )
        )
    );


    my $person = $class_person->new
    (
        name => 'Benigno Millares',
        alias => 'benny',
        dateOfBirth => '10/24/1970'
    );

    # 3: does our object have the right type?
    isa_ok
    (
        $person, $class_person,
        'new Person'
    );


    # 4: is the name attribute set on creation? (also tests accessor)
    is
    (
        $person->name, 'Benigno Millares',
        'Person->name'
    );

    # 5: is the alias attribute set on creation? (also tests accessor)
    is
    (
        $person->alias, 'benny',
        'Person->alias'
    );

    # 6: is the dateOfBirth attribute set on creation? (also tests accessor)
    is
    (
        $person->dateOfBirth, '10/24/1970',
        'Person->dateOfBirth'
    );
}


{

    my $person = $class_person->new
    (
        name => 'Benigno Millares',
        dateOfBirth => '10/24/1989'
    );

    # changing it to something else after instantiating object
    $person->name('Daniel Gonzales');
    $person->alias('danielito');

    # 7: does the mutator for the name attribute work?
    is
    (
        $person->name, 'Daniel Gonzales',
        'Person->name'
    );

    # 8: does the mutator for the alias attribute work?
    is
    (
        $person->alias, 'danielito',
        'Person->alias'
    );

    # 9: obviously the dateOfBirth attribute should still have the same value
    is
    (
        $person->dateOfBirth, '10/24/1989',
        'Person->dateOfBirth'
    );
}


# testing negatives
{
    my $person = eval
    {
        $class_person->new
        (
            name => 'Benigno Millares',
            dateOfBirth => '10/24/1970'
        )
    };

    # 10: is the alias attribute optional?
    ok
    (
        !$@,
        'Person->alias is optional attribute.'
    );
}

{
    my $person = eval
    {
        $class_person->new
        (
            alias => 'benny',
            dateOfBirth => '10/24/1970'
        );
    };

    # 11: is the name attribute required?
    like
    (
        $@, qr/name/,
        'Person->name is required attribute.'
    );
}

{
    my $person = eval
    {
        $class_person->new
        (
            name => 'Benigno Millares',
            alias => 'benny',
        );
    };

    # 12: is the dateOfBirth attribute required?
    like
    (
        $@, qr/dateOfBirth/,
        'Person->dateOfBirth is required attribute.'
    );
}


{
    my $person = eval
    {
        $class_person->new
        (
            name => 'Benigno Millares',
            alias => 'benny',
            dateOfBirth => '1920/013/01',
        );
    };

    # 13: is the dateOfBirth attribute checked for valid dates?
    like
    (
        $@, qr/dateOfBirth/,
        'Bonus: Person->dateOfBirth format exception.'
    );
}

{
    my $person = eval
    {
        $class_person->new
        (

            name => 'Benigno Millares',
            alias => 'benny',
            dateOfBirth => '10/24/1970',
            invalidAttribute => 1
        );
    };

    # 14: are extra (unknown) attributes caught as an error?
    like
    (
        $@, qr/invalidAttribute/,
        'Bonus: Person->invalidAttribute not a valid attribute exception.'
    );
}


{
    eval
    {
        $class_person->new
        (
            name => [qw(Benigno Millares)],
            alias => 'benny',
            dateOfBirth => '10/24/1970',
        );
    };

    # 15: is the type of the name attribute enforced?
    like
    (
        $@, qr/name/,
        'Bonus: Person->name should have type string.'
    );
}


