# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl OOPerlTest-Personal.t'

#########################

use Test::More tests => 16;


my $class_person = 'OOPerlTest::Personal::Person';
my $class_workUnit = 'OOPerlTest::Personal::WorkUnit';
my $class_employee = 'OOPerlTest::Personal::Person::Employee';
my $class_manager = 'OOPerlTest::Personal::Person::Employee::Manager';
eval qq{ use $class_workUnit };

# 1: can we load our class?
use_ok($class_manager);


# testing positive
{
    # 2: are all attributes (including inherited ones) available as methods?
    can_ok
    (
        $class_manager,
        (
            qw(
                name
                alias
                dateOfBirth
                hireDate
            )
        )
    );

    my $manager = $class_manager->new
    (
        name => 'Benigno Millares',
        alias => 'benny',
        dateOfBirth => '10/24/1970',
        hireDate => '01/01/2010'
    );

    # 3,4,5: does our object have the right type (including inheritance)?
    isa_ok $manager, $class_manager, 'new Manager';
    isa_ok $manager, $class_employee, 'new Manager';
    isa_ok $manager, $class_person, 'new Manager';

    # 6: is the name attribute set on creation? (also tests accessor)
    is
    (
        $manager->name, 'Benigno Millares',
        'Manager->name holds its value'
    );

    # 7: is the alias attribute set on creation? (also tests accessor)
    is
    (
        $manager->alias, 'benny',
        'Manager->alias holds its value'
    );

    # 8: is the dateOfBirth attribute set on creation? (also tests accessor)
    is
    (
        $manager->dateOfBirth, '10/24/1970',
        'Manager->dateOfBirth holds its value'
    );

    # 9: is the hireDate attribute set on creation? (also tests accessor)
    is
    (
        $manager->hireDate, '01/01/2010',
        'Manager->hireDate holds its value'
    );

}

{
    # create a manager
    my $manager = $class_manager->new
    (
        name => 'Rafael Rodriguez',
        alias => 'rafe',
        dateOfBirth => '10/24/1970',
        hireDate => '01/01/2010'
    );

    # gather some people
    my @humans =
    (
        $class_person->new
        (
            name => 'Jack Smith',
            alias => 'Jackie',
            dateOfBirth => '07/17/1980',
        ),
        $class_person->new
        (
            name => 'Jill Brown',
            dateOfBirth => '09/03/1974',
        ),
        $class_person->new
        (
            name => 'James Resputin',
            dateOfBirth => '12/11/1969',
        ),
        $class_person->new
        (
            name => 'Maria Flores',
            alias => 'slick',
            dateOfBirth => '01/27/1979'
        )
    );

    # 10: basic sanity check [all humans are Person's]
    is
    (
        scalar(grep { $_->isa($class_person) } @humans), scalar(@humans),
        'All persons are ready for jobs.'
    );



    # let's start hiring
    @humans = map { $manager->Hire($_) } @humans;

    # 11: did all our Person's turn into Employee's?
    is
    (
        scalar(grep { $_->isa($class_employee) } @humans), scalar(@humans),
        'All persons where hired'
    );

    # create some work
    my @workUnits =
    (
        (
            map
            {
                $class_workUnit->new
                (
                    project => 'Making doughnuts',
                    task => 'cook batch ' . $_,
                    status => 'NotStarted',
                    payload => <<END,
01/13/2009 some event occurred
04/29/2009 some other event occurred
Kilroy was here!
an unusual event occurred on 06/09/2009
10/31/2009 was Halloween
END
                )
            } (1 .. 50)
        ),
        (
            map
            {
                $class_workUnit->new
                (
                    project => 'Making doughnuts',
                    task => 'cook batch ' . $_,
                    status => 'NotStarted',
                    payload => <<END,
20090113 some event occurred
20090429 some other event occurred
Kilroy was here!
an unusual event occurred on 06/09/2009
20091031 was Halloween
END
                )
            } (1 .. 10)
        ),
        (
            map
            {
                $class_workUnit->new
                (
                    project => 'Making doughnuts',
                    task => 'cook batch ' . $_,
                    status => 'NotStarted',
                )
            } (1 .. 10)
        )
    );

    # get to work
    @workUnits =
        map
        {
            $manager->Work($_)
        } @workUnits;


    # check results
    # 12: make sure we have the proper numbers of Completed and Rejected
    is
    (
        $manager->WorkUnitsProcessed, 'Completed: 50, Rejected: 20',
        'All work is successfully completed.'
    );

    # let's now fire them all
    @humans = map { $manager->Fire($_) } @humans;

    # 13: did firing them make them all cease to be Employee's?
    is
    (
        scalar(grep { $_->isa($class_employee) } @humans), 0,
        'Bonus: All employees were fired.'
    );



    # let's re-hire them
    @humans = map { $manager->Hire($_) } @humans;

    # 14: did rehiring them turn them all back into Employee's again?
    is
    (
        scalar(grep { $_->isa($class_employee) } @humans), scalar(@humans),
        'All persons where re-hired'
    );


    # now let's have them quit
    @humans = map { $_->Quit } @humans;

    # 15: make sure none of our Person's are still Employee's after Quit'ting
    is
    (
        scalar(grep { $_->isa($class_employee) } @humans), 0,
        'All quit succesfully'
    );

}


{
    # create a manager
    my $manager = $class_manager->new
    (
        name => 'Rafael Rodriguez',
        alias => 'rafe',
        dateOfBirth => '10/24/1970',
        hireDate => '01/01/2010'
    );

    # gather some people
    my @humans =
    (
        $class_person->new
        (
            name => 'Jack Smith',
            alias => 'Jackie',
            dateOfBirth => '07/17/1980',
        ),
        $class_person->new
        (
            name => 'Jill Brown',
            dateOfBirth => '09/03/1974',
        ),
        $class_person->new
        (
            name => 'James Resputin',
            dateOfBirth => '12/11/1969',
        ),
        $class_person->new
        (
            name => 'Maria Flores',
            alias => 'slick',
            dateOfBirth => '01/27/1979'
        )
    );

    # hire them all
    @humans = map { $manager->Hire($_) } @humans;

    # create some work
    my @workUnits =
        map
        {
            $class_workUnit->new
            (
                project => 'Making doughnuts',
                task => 'cook batch ' . $_,
                status => (sort { rand 1 <=> 0.5 } qw(Rejected Completed))[0],
            )
        } (1 .. 50);

    # get to work
    @workUnits =
        map
        {
            $manager->Work($_)
        } @workUnits;

    # check results
    # 16: work units that were already Completed/Rejected don't get processed again
    is
    (
        $manager->WorkUnitsProcessed, 'Completed: 0, Rejected: 0',
        'Bonus: No work was performed.'
    );

}


